from motherhamster.displayGL import *

class TilingTexture:
    """A class for a tiling texture in openGL - it solves power of 2 issues"""
    
    def __init__(self, image):
        """Initialize the tiling texture from an ImageAsset
           the surface must be a power of 2
           currently, hotspot is ignored (cause I don't know what it should mean"""
        surface = image.surface        
        self.width, self.height = surface.get_size()
        assert((self.width & (self.width - 1)) == 0 and (self.height & (self.height - 1)) == 0, "surface must be a power of 2 to be used for tiling")

        self.texture_id = glGenTextures(1)            
        glBindTexture(GL_TEXTURE_2D, self.texture_id)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR)
        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR)
        pixels = GetLightOpacityStringForSurface(surface)
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, self.width, self.height, 0, GL_RGBA, GL_UNSIGNED_BYTE, pixels)

    def __del__(self):
        glDeleteTextureIfSafeTo(self.texture_id)                            

def quad(self, point1, point2, point3, point4, color=(1,1,1,1), image=None):
    """Draws an arbitrary quad, either solid colored or using an image"""
    glLoadIdentity()
    glColor4f(color[0], color[1], color[2], color[3])
    if image != None:
        if (not hasattr(image, "GLtexture")):
            self.pack_image(image)
        if image.blend_mode != ImageAsset.blend_modes.alpha_blend:
            glBlendFunc(self.src_blend[image.blend_mode], self.dest_blend[image.blend_mode])
        glBindTexture(GL_TEXTURE_2D, image.GLtexture.texture_id)
        glEnable(GL_TEXTURE_2D)
        glBegin(GL_QUADS)
        glTexCoord2d(image.GLtexels[0], image.GLtexels[1]); glVertex2f(point1[0], point1[1])
        glTexCoord2d(image.GLtexels[0], image.GLtexels[3]); glVertex2f(point2[0], point2[1])
        glTexCoord2d(image.GLtexels[2], image.GLtexels[3]); glVertex2f(point3[0], point3[1])
        glTexCoord2d(image.GLtexels[2], image.GLtexels[1]); glVertex2f(point4[0], point4[1])
        glEnd()
        if image.blend_mode != ImageAsset.blend_modes.alpha_blend:
            glBlendFunc(self.src_blend[ImageAsset.blend_modes.alpha_blend], self.dest_blend[ImageAsset.blend_modes.alpha_blend])
    else:
        glDisable(GL_TEXTURE_2D)
        glColor4f(color[0], color[1], color[2], color[3])
        glBegin(GL_QUADS)
        glVertex2f(point1[0], point1[1])
        glVertex2f(point2[0], point2[1])
        glVertex2f(point3[0], point3[1])
        glVertex2f(point4[0], point4[1])
        glEnd()

def textured_line_segment(self, image, start, end, scale=(1,1)):
    if (not hasattr(image, "GLtile")):
        image.GLtile = TilingTexture(image)
    xdiff = end[0] - start[0]
    ydiff = end[1] - start[1]
    length = math.sqrt(float(xdiff*xdiff + ydiff*ydiff))
    length_in_texels = length/(image.width*scale[0])
    perp_xdiff = -(ydiff/length)*(image.height*scale[1]/2.0)
    perp_ydiff = (xdiff/length)*(image.height*scale[1]/2.0)
    
    glLoadIdentity()
    glBindTexture(GL_TEXTURE_2D, image.GLtile.texture_id)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT)
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP)    
    glEnable(GL_TEXTURE_2D)
    glBegin(GL_QUADS)
    glTexCoord2d(0, 1)
    glVertex2f(start[0] + perp_xdiff, start[1] + perp_ydiff)
    glTexCoord2d(length_in_texels, 1)
    glVertex2f(end[0] + perp_xdiff, end[1] + perp_ydiff)
    glTexCoord2d(length_in_texels, 0)
    glVertex2f(end[0] - perp_xdiff, end[1] - perp_ydiff)
    glTexCoord2d(0, 0)
    glVertex2f(start[0] - perp_xdiff, start[1] - perp_ydiff)
    glEnd()
    

Display.quad = quad
Display.textured_line_segment = textured_line_segment