#!/usr/bin/env python
# (C) Copyright 2008 Brian Fisher, Zach Young, James Paige
# This is Free Software released under the terms of the GPL
# Created for the Pyweek May 2008 competition: "ROBOTS!"

from locals import *
from motherhamster.cheekpouches import GetPouch
from motherhamster.fooddish import AssetPath

from city import CityBlock, BuildingStyle
from movement import Compass
from grabber import Grabber
from slantedrect import SlantedRect

import addExtrasToDisplayGL

#------------------------------------------------------------------------------

class Editor(HamsterWheel):

  font = AssetRef(GlyphSet, "default")
  cursor = AssetRef(ImageAsset, "laser_sight")
  aiming_glow = AssetRef(ImageAsset, "laser_sight_glow")
  
  def __init__(self):
    HamsterWheel.__init__(self, "City Block Editor (ROBOT!)")
    self.camera = vec2d(0,0) + 60
    self.compass = Compass()
    self.camera_velocity = vec2d(0,0)
    self.typer = GlyphTyper(self.display, self.font)
    self.editblock = None
    self.changes = False
    self.current_name = ""
    self.cycle_block()
    self.grabber = Grabber()

  def process_sdl_event(self, event):
    HamsterWheel.process_sdl_event(self, event)
    if event.type == KEYDOWN:
      for d in self.compass:
        if d.pressed(event):
          d.set(self.camera_velocity, 1)
    if event.type == KEYUP:
      for d in self.compass:
        if d.pressed(event):
          d.set(self.camera_velocity, 0)
      if event.key == K_ESCAPE:
        self.quit()
      elif event.key == K_PERIOD:
        self.cycle_block(1)
      elif event.key == K_COMMA:
        self.cycle_block(-1)
      elif event.key == K_BACKSPACE:
        self.revert_changes()
      elif event.key == K_s and pressing_ctrl(event):
        self.save_changes()
      elif event.key == K_LEFTBRACKET:
        self.cycle_building(-1)
      elif event.key == K_RIGHTBRACKET:
        self.cycle_building(1)
    if event.type == MOUSEBUTTONUP:
      if event.button == 4:
        self.cycle_building(1)
      if event.button == 5:
        self.cycle_building(-1)
    self.grabber.process_sdl_event(event)

  def draw(self):
    self.display.erase()
    if self.editblock == None:
      return
    self.editblock.draw_at(self.display, vec2d(0,0))
    things = []
    self.editblock.get_buildings_at(things, vec2d(0,0))
    things.sort(reverse=True, key=lambda x : x.z_sort_order())
    slant = SlantedRect()
    for thing in things:
      thing.draw(self.display)
      slant.pos = thing.object.style.obstruct_pos
      slant.size = thing.object.style.obstruct_size
      (bl, br, tr, tl) = slant.corners(thing.pos)
      self.display.quad(bl, br, tr, tl, [0,0,1,.3])
      (bl, br, tr, tl) = expand_point(thing.pos, 1.5)
      self.display.quad(bl, br, tr, tl, [1,1,0,.8])
    screen_half = vec2d(self.screen_width, self.screen_height) / 2
    where = vec2d(0,0) - (self.camera - screen_half)
    (bl, br, tr, tl) = expand_point(where, 1.5)
    self.display.quad(bl, br, tr, tl, [0,1,0,.8])

    self.draw_ui(self.display)

  def draw_ui(self, display):
    asset_id = self.editblock.asset_metadata['path']
    self.typer.draw(asset_id, [0, self.screen_height-30])
    if self.changes:
      self.typer.draw("changed", [0,5])
    mouse = self.mouse
    display.render(self.cursor, mouse.pos)
    display.render(self.aiming_glow, mouse.pos)

  def update(self, seconds):
    for d in self.compass:
      if d.get(self.camera_velocity) > 0:
        d.go(self.camera, 4)
    self.grabber.update()

  def nearest_building(self, pos):
    # this is used by grabber
    nearest = vec2d(self.screen_width, self.screen_height) * 10
    found = None
    things = []
    self.editblock.get_buildings(things, vec2d(0,0))
    for thing in things:
      dist = self.mouse.pos.get_distance(thing.pos)
      if dist < nearest:
        nearest = dist
        found = thing.object
    return found

  def nearest_building_slot(self):
    # this is used by cycle_building
    nearest = vec2d(self.screen_width, self.screen_height) * 10
    found = None
    things = []
    screen_half = vec2d(self.screen_width, self.screen_height) / 2
    for i in xrange(len(self.editblock.buildings)):
      building = self.editblock.buildings[i]
      where = vec2d(0,0) - self.camera + screen_half + building.pos
      dist = self.mouse.pos.get_distance(where)
      if dist < nearest:
        nearest = dist
        found = i
    return found

  def save_changes(self):
    if self.changes:
      self.changes = False
      if not self.editblock.asset_save():
        print self.editblock, "is not writable!"
      for building in self.editblock.buildings:
        building.style.asset_save()

  def revert_changes(self):
    self.changes = False
    GetPouch(CityBlock).RevertAsset(self.current_name)

  def load_cityblock(self, asset_id):
    self.changes = False
    self.current_name = str(asset_id)
    print "Loading:", self.current_name
    self.editblock = Disgorge(CityBlock, self.current_name)

  def quit(self):
    self.save_changes()
    HamsterWheel.quit(self)

  def cycle_block(self, direction=1):
    print "cycle_block"
    self.save_changes()
    autogenerate_missing_buildingstyles()
    all = AvailableAssetIDs(CityBlock)
    if len(all) == 0:
      print "no CityBlock assets found"
      return
    if self.editblock == None:
      self.load_cityblock(all[0])
      return
    for i in xrange(len(all)):
      asset_id = str(all[i])
      print asset_id, self.current_name
      if asset_id == self.current_name:
        index = (i + direction) % len(all)
        self.load_cityblock(all[index])
        return
    print "cycle_block ended with no load_cityblock"

  def cycle_building(self, direction=1):
    autogenerate_missing_buildingstyles()
    all_ids = AvailableAssetIDs(BuildingStyle)
    all = []
    for id in all_ids:
      if not id[:-1].endswith("_hit"):
        all.append(id)
    if len(all) == 0:
      print "no BuildingStyle assets found"
      return
    slot = self.nearest_building_slot()
    building = self.editblock.buildings[slot]
    seek_id = building.style.asset_metadata["path"]
    for i in xrange(len(all)):
      asset_id = str(all[i])
      print asset_id, seek_id
      if asset_id == seek_id:
        index = (i + direction) % len(all)
        building.style = all[index]
        self.changes = True
        return
    print "cycle_building ended with no building change"

#------------------------------------------------------------------------------

def autogenerate_missing_buildingstyles():
  all_images = AvailableAssetIDs(ImageAsset)
  all_buildingstyles = AvailableAssetIDs(BuildingStyle)
  for image in all_images:
    name = str(image)
    if name.startswith('buildings/') or name.startswith('trees/'):
      if not name in all_buildingstyles:
        pouch = GetPouch(BuildingStyle)
        path_name = name + pouch.AssetExtensions()[0]
        path = AssetPath(path_name, image.root)
        style = pouch.CreateNewAsset(path)
        style.asset_set_property("image", name)
        style.asset_save()

#------------------------------------------------------------------------------

def pressing_ctrl(event):
  return (event.mod & (KMOD_LCTRL | KMOD_RCTRL)) > 0

#------------------------------------------------------------------------------

def expand_point(point, amount=1):
  return (point + [-amount, -amount], point + [amount, -amount], point + [amount, amount], point + [-amount, amount])

#------------------------------------------------------------------------------

if __name__ == "__main__":
  editor = Editor()
  editor.run_game()
