# (C) Copyright 2008 Brian Fisher, Zach Young, James Paige
# This is Free Software released under the terms of the GPL
# Created for the Pyweek May 2008 competition: "ROBOTS!"

from locals import *
from random import randint

from city import CityBlock

#------------------------------------------------------------------------------

class CustomerData(AssetBase):
    help_image = AssetRef(ImageAsset, "interface/balloon_help")
    thank_you_image = AssetRef(ImageAsset, "interface/balloon_great_service")
    thank_you_time = AssetFloat(5.0)
    
#------------------------------------------------------------------------------

class Customer(object):

  style = AssetRef(CustomerData)
  
  def __init__(self):
    self.name = "Joe Bloe"
    self.time_limit = 15 * 60.0 # 15 minutes?
    self.time_on_hold = 0.0
    self.thank_you_time = .0
    self.where = vec2d(2,1)

  def update(self, seconds):
    self.time_on_hold += seconds
    if self.thank_you_time > .0:
        self.thank_you_time -= seconds
        if self.thank_you_time < .0:
            self.thank_you_time = .0
            self.time_on_hold = 0
            self.where += vec2d(randint(-5, 5), randint(-5,5))
    else:
        game = HamsterWheel.the_game
        if (game.robot.pos - self.pixel_pos()).length < 250:
            self.thank_you_time = self.style.thank_you_time

  def pixel_pos(self):
    block_offset = CityBlock.size + CityBlock.street_size
    result = self.where * block_offset
    if self.where.y % 2 == 1:
      result.x -= block_offset.x / 2
    return result

  def draw(self, display):
    game = HamsterWheel.the_game
    if self.thank_you_time > .0:
      display.render(self.style.thank_you_image, self.pixel_pos() - game.camera)
    else:
      display.render(self.style.help_image, self.pixel_pos() - game.camera)
      
  def satisfaction(self, out_of=5):
    return out_of - int(out_of / self.time_limit * self.time_on_hold)

  def time_string(self):
    return "%2d:%02d" % (self.time_on_hold / 60, self.time_on_hold % 60) 

  def where_in_pixels(self):
    size = CityBlock.size + CityBlock.street_size
    return self.where * size

__BrokenCustomer = CustomerData()
RegisterAssetExtension(CustomerData, "customer", __BrokenCustomer)
