# (C) Copyright 2008 Brian Fisher, Zach Young, James Paige
# This is Free Software released under the terms of the GPL
# Created for the Pyweek May 2008 competition: "ROBOTS!"

from locals import *

#------------------------------------------------------------------------------

class Grabber(object):
  """This object is used for interactive repositioning of buildings"""

  drag = Enum("move", "size")

  def __init__(self):
    # Triggers the mouse to grab a building
    self._trigger = False
    # Keeps track of what building is currently being grabbed
    self._object = None
    self._dragging_all = False
    # Keeps track of grabbing movement
    self._old = vec2d(0,0)
    self._at = vec2d(0,0)
    # set the update method
    self.update = self.update_none
    # set the drag mode
    self.drag = Grabber.drag.move

  def process_sdl_event(self, event):
    if event.type == MOUSEBUTTONDOWN:
      if event.button == 1:
        self._trigger = True
        self.update = self.update_one
      if event.button == 2:
        self._trigger = True
        self.update = self.update_all
    elif event.type == MOUSEBUTTONUP:
      if event.button == 1:
        self._object = None
        self.update = self.update_none
      if event.button == 2:
        self._dragging_all = False
        self.update = self.update_none
    if event.type == KEYDOWN:
      if event.key in (K_LSHIFT, K_RSHIFT):
        self.drag = Grabber.drag.size
    if event.type == KEYUP:
      if event.key in (K_LSHIFT, K_RSHIFT):
        self.drag = Grabber.drag.move

  def update_none(self):
    pass

  def update_one(self):
    game = HamsterWheel.the_game
    if self._trigger:
      self._trigger = False
      self._object = game.nearest_building(game.mouse.pos)
      if self._object != None:
        if self.drag == Grabber.drag.move:
          self._old = vec2d(self._object.pos)
        elif self.drag == Grabber.drag.size:
          self._old = vec2d(self._object.style.obstruct_size)
        self._at = vec2d(game.mouse.pos)
    if self._object != None:
      if self.drag == Grabber.drag.move:
        new_pos = self._old + (game.mouse.pos - self._at)
        if self._object.pos != new_pos:
          self._object.pos = new_pos
          game.changes = True
      elif self.drag == Grabber.drag.size:
        new_size = self._old + (game.mouse.pos - self._at)
        new_size.x = abs(new_size.x)
        new_size.y = abs(new_size.y)
        if self._object.style.obstruct_size != new_size:
          self._object.style.obstruct_size = new_size
          game.changes = True

  def update_all(self):
    game = HamsterWheel.the_game
    if self._trigger:
      self._trigger = False
      self._dragging_all = True
      self._old = vec2d(game.mouse.pos)
    if self._dragging_all:
      for building in game.editblock.buildings:
        new_pos = building.pos + (game.mouse.pos - self._old)
        if new_pos != building.pos:
          building.pos = new_pos
          game.changes = True
      self._old = vec2d(game.mouse.pos)
