#!/usr/bin/env python
# (C) Copyright 2008 Brian Fisher, Zach Young, James Paige
# This is Free Software released under the terms of the GPL
# Created for the Pyweek May 2008 competition: "ROBOTS!"

import pygame
from locals import *
from city import CityMap
from movement import Compass
from grabber import Grabber
from slantedrect import SlantedRect
from customer import Customer
from random import choice

import addExtrasToDisplayGL

import customerservice

#------------------------------------------------------------------------------

class Game(HamsterWheel):

  font = AssetRef(GlyphSet, "default")

  def __init__(self):
    HamsterWheel.__init__(self, "Customer Service ROBOT!")
    self.camera = vec2d(0,0)
    self.map = CityMap()
    self.compass = Compass()
    self.camera_velocity = vec2d(0,0) # this is probably not how we want to do camera movement
    self.robot = customerservice.Robot()
    self.typer = GlyphTyper(self.display, self.font)
    self.debug_text = ""
    self.score_damage = 0.00
    self.caller = Customer()
    self.effects = []
    music = Disgorge(SoundAsset, "sounds/music")
    music.set_as_music()
    self.recordings = []
    self.current_recording = None
    self.get_all_recordings()
    self.recording_countdown = customerservice.SoundCountdown(1000)

  def draw(self):
    bottom_left = self.camera + (0,-self.map.style.max_building_height_hack)
    top_right = self.camera + (self.display.width, self.display.height)

    self.map.draw_ground(self.display, bottom_left, top_right)
    
    things = game.map.get_buildings_intersecting(bottom_left, top_right)
    things.append(self.robot)
    for effect in self.effects:
        things.append(effect)
    things.sort(reverse=True, key=lambda x : x.z_sort_order())
    for thing in things:
      thing.draw(self.display)
    
    self.caller.draw(self.display)
    self.robot.draw_ui(self.display)
   
  def draw_debug(self):
    # would be good to draw some collision stuff here?
    self.typer.draw(self.debug_text, (0,10))
    
    slant = SlantedRect()
    bottom_left = self.camera + (0,-self.map.style.max_building_height_hack)
    top_right = self.camera + (self.display.width, self.display.height)
    things = game.map.get_buildings_intersecting(bottom_left, top_right)
    for thing in things:
      if thing.object.still_exists():
        slant.pos = thing.object.style.obstruct_pos
        slant.size = thing.object.style.obstruct_size
        (bl, br, tr, tl) = slant.corners(thing.pos)
        self.display.quad(bl, br, tr, tl, [0,0,1,.3])
    
    self.robot.draw_debug(self.display)

  def update(self, seconds):
    if self.keys.pressed(K_ESCAPE):
        self.quit()
        return
      
    self.robot.update(seconds)
    self.caller.update(seconds)
    
    dead_effect_list = []
    for effect in self.effects:
        effect.update(seconds)
        if effect.is_finished():
            dead_effect_list.append(effect)
    for effect in dead_effect_list:
        self.effects.remove(effect)
    
    if self.current_recording == None:
      if self.recording_countdown():
        self.play_recording()
    else:
      if self.current_recording.get_busy() == False:
        music_channel = pygame.mixer.Channel(0)
        music_channel.unpause()
        self.current_recording = None

  def play_recording(self):
    if len(self.recordings) == 0:
      self.get_all_recordings()
    if len(self.recordings) == 0:
      return
    music_channel = pygame.mixer.Channel(0)
    music_channel.pause()
    sound = choice(self.recordings)
    self.current_recording = sound.play()
    self.recordings.remove(sound)
    
  def get_all_recordings(self):
    self.recordings = []
    sound_paths = AvailableAssetIDs(SoundAsset)
    for sound_path in sound_paths:
      sound_name = str(sound_path)
      if sound_name.startswith("sounds/R_"):
        self.recordings.append(Disgorge(SoundAsset, sound_name))


#------------------------------------------------------------------------------

if __name__ == "__main__":
  game = Game()
  game.run_game()
