"""
displays debug images registered for stuff
(C) Copyright 2007 Hamster Republic Productions
"""

########################################################################
from motherhamster import editorinfo

import safe_wxversion
import wx
import pygame

########################################################################
class DebugImageBrowser(wx.Frame):
    """ A window that lets you browse the different asset types
        """
    _Browser = None
    def __Activate():
        if (DebugImageBrowser._Browser == None):
            DebugImageBrowser._Browser = DebugImageBrowser(None)
            DebugImageBrowser._Browser.Show()
        else:
            DebugImageBrowser._Browser.Raise()
    Activate = staticmethod(__Activate)
    
    def __init__(self, parent):
        wx.Frame.__init__(self, parent, -1, 'DebugImageBrowser', size = (200, 400),
                          style=wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)
        
        self.panel = wx.Panel(self, -1)
        
        self.ImageTypeChoice = wx.Choice(self.panel, -1, choices = editorinfo.GetImageSources().keys())
        self.Bind(wx.EVT_CHOICE, self.OnChoice, self.ImageTypeChoice)

        self.image_index = 0

        self.PrevButton = wx.Button(self.panel, -1, "Prev")
        self.PrevButton.Enable(False)
        self.Bind(wx.EVT_BUTTON, self.PrevImage, self.PrevButton)

        self.NextButton = wx.Button(self.panel, -1, "Next")
        self.NextButton.Enable(False)
        self.Bind(wx.EVT_BUTTON, self.NextImage, self.NextButton)
        
        self.CountText = wx.StaticText(self.panel, label = "000/000")
        
        buttonbox = wx.BoxSizer(wx.HORIZONTAL)
        buttonbox.Add((0,0), 1)
        buttonbox.Add(self.PrevButton, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        buttonbox.Add((0,0), 1)
        buttonbox.Add(self.CountText, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        buttonbox.Add((0,0), 1)
        buttonbox.Add(self.NextButton, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        buttonbox.Add((0,0), 1)

        self.ImagePanel = DebugImagePanel(self.panel)
        self.InfoText = wx.StaticText(self.panel)
        
        verticalbox = wx.BoxSizer(wx.VERTICAL)
        verticalbox.Add(self.ImageTypeChoice, 0, wx.EXPAND)
        verticalbox.Add(buttonbox, 0, wx.EXPAND)
        verticalbox.Add(self.ImagePanel, 0, wx.ALL)
        verticalbox.Add(self.InfoText, 0, wx.ALIGN_CENTER)
        
        self.top_sizer = verticalbox
        
        self.panel.SetSizerAndFit(verticalbox)
        self.SetSize(self.GetBestSize())
        
    #---------------------------------------------
    def OnChoice(self, event):
        self.source_name = event.GetString()
        self.image_index = 0
        self.SetImageData()
                            
    #---------------------------------------------
    def PrevImage(self, event):
        self.image_index -= 1
        self.SetImageData()
        
    #---------------------------------------------
    def NextImage(self, event):
        self.image_index += 1
        self.SetImageData()
        
    #---------------------------------------------
    def SetImageData(self):
        source = editorinfo.GetImageSources()[self.source_name]
        num_items = source.GetNumItems()
        self.CountText.SetLabel("%d/%d" % (self.image_index + 1, num_items))
        self.InfoText.SetLabel(source.GetInfo(self.image_index))
        self.NextButton.Enable(self.image_index < num_items - 1)
        self.PrevButton.Enable(self.image_index > 0)
        self.ImagePanel.SetImage(source.GetImage(self.image_index))
        self.top_sizer.SetSizeHints(self)
        self.Fit()
        self.SetSize(self.GetMinSize())        
    
    #---------------------------------------------
    def KeepRootOpen(self, event):
        item = event.GetItem()
        if (item == self.root or self.AssetDirectoryList.GetItemParent(item) == self.root):
            event.Veto()
        
    #---------------------------------------------
    def OnCloseWindow(self, event):
        self.Destroy()
        if (DebugImageBrowser._Browser == self):
            DebugImageBrowser._Browser = None

########################################################################
class DebugImagePanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1, style = wx.SUNKEN_BORDER)
        self.bitmap = None
        self.Bind(wx.EVT_PAINT, self.OnPaint)

    def SetImage(self, image):
        image_string = pygame.image.tostring(image, "RGB")
        imgWx = wx.EmptyImage(image.get_width(), image.get_height())
        self.SetClientSize((image.get_width(), image.get_height()))
        self.SetMinSize(self.GetSize())
        imgWx.SetData(image_string)
        self.bitmap = imgWx.ConvertToBitmap()
        self.Refresh()
    
    def OnPaint(self, evt):
        dc = wx.PaintDC(self)
        dc.SetBackground(wx.Brush("WHITE"))
        dc.Clear()
        if self.bitmap != None:
            dc.DrawBitmap(self.bitmap, 0, 0, True)

if __name__ == '__main__':
    def GetNumItems():
        return 4

    def GetInfo(index):
        return "This is info text for item %d!!" % index

    def GetImage(index):
        index += 1
        size = [max((index*1777) % 800, 20), max((7937 *index) % 300, 20)]
        print size
        surface = pygame.Surface(size, 0, 24)
        color = ((index * 937) % 255, (index * 7133) % 255, (index * 64) % 255, 255)
        print color
        print surface.fill(color)
        return surface

    editorinfo.RegisterImageSource("test source", GetNumItems, GetImage, GetInfo)
    
    class _TestApp(wx.App):
        def __init__(self):
            self.Window = None
            wx.App.__init__(self, redirect=False, useBestVisual=True)
            
        def OnInit(self):
            DebugImageBrowser.Activate()
            return True
        
    app = _TestApp()
    app.MainLoop()
