"""
Implementation of how to render to the display for SDL stuff

(C) Copyright 2005 Hamster Republic Productions
"""

import pygame
from motherhamster import imageassets
from motherhamster.mathobj import vec2d

########################################################################
class CachedTransform:
    """a python object wrapper for a pygame surface & a hotspot
       these are so the SDL backend can be fairly efficient about
       not resizing an image the same way over and over again"""
    def __init__(self, raw_surface, hotspot, scale, angle_degrees, framenum):
        self.surface = pygame.transform.rotozoom(raw_surface, angle_degrees, scale)
        self.hotspot = hotspot
        self.framenum = framenum
        self.hotspot = hotspot - [raw_surface.get_width()/2.0, raw_surface.get_height()/2.0]
        self.hotspot.rotate(angle_degrees)
        self.hotspot *= scale
        self.hotspot += [self.surface.get_width()/2.0, self.surface.get_height()/2.0]
        self.__memory_estimate = imageassets.CountSurfaceMemory(raw_surface)

    def __del__(self):
        if (imageassets.SubtractSurfaceMemory != None):
            imageassets.SubtractSurfaceMemory(self.__memory_estimate)
        
########################################################################
class AssetImageCache:
    def GetCachedImage(self, image, scale, angle_degrees):
        """ Just a function to store some dead code in..."""
        # CHECK: might be nice to have this angle thing in general purpose angle helper stuff
        if abs(scale - 1.0) <= self.cache_scale_step and abs(angle_degrees) <= self.cache_angle_step:
            return ImageInstance(self.__rawimage, self.__hotspot)
        else:       
            entry = None 
            for entryToTest in self.__weak_cache:
                if (abs(entryToTest[1] - scale) <= self.cache_scale_step and abs(entryToTest[0] - angle_degrees) <= self.cache_angle_step):
                    if entryToTest[2]() != None:
                        return entryToTest[2]()
                    else:
                        entry = entryToTest
                        break
            #CHECK: should measure and log time spent transforming images...
            if (entry == None):
                entry = [angle_degrees, scale, None, 0]
                self.__weak_cache.append(entry)
            entry[3] = imageassets.CountSurfaceMemory(surface)
            instance = ImageInstance(surface, hotspot)
            entry[2] = weakref.ref(instance, self.__lose_instance)
            return instance

    def __lose_instance(self, weakref):
        for entryToTest in self.__weak_cache:
            if entryToTest[2] == weakref:
                _SubtractSurfaceMemory(entryToTest[3])
                self.__weak_cache.remove(entryToTest)
                return
        #CHECK: need some error logging system if we don't find weak-ref in cache (something that can be dumped into editor window)
    
########################################################################
class Display:
    """pygame window with openGL"""
    def __init__(self):
        self.surface = pygame.display.get_surface()
        self.frame_num = 0
        self.__weak_cache = []
        self.cache_angle_step = 1
        self.cache_scale_step = .01
    
    def render(self, request):
        surface, hotspot = self._get_cached_transform(request)
        pos = vec2d(request.position.x - hotspot.x, (self.maxy - request.position.y) - (surface.get_height() - hotspot.y))
        if (request.src_rect != None):
            src_rect = pygame.Rect(request.src_rect.left*request.scale, (request.image.height - (request.src_rect.top + request.src_rect.height))*request.scale,
                                   request.src_rect.width*request.scale, request.src_rect.height*request.scale)
            self.surface.blit(surface, pos, src_rect)
        else:
            self.surface.blit(surface, pos)
    
    def _get_cached_transform(self, request):
        cache_angle = ((request.angle + 180) % 360) - 180
        if abs(request.scale - 1.0) <= self.cache_scale_step and abs(cache_angle) <= self.cache_angle_step:
            return request.image.surface, request.image.hotspot
        #TODO: do the cache stuff
        cached = CachedTransform(request.image.surface, request.image.hotspot, request.scale, cache_angle, self.frame_num)
        return cached.surface, cached.hotspot
    
    def set_mode(self, width, height, fullscreen = False):
        flags = pygame.DOUBLEBUF
        if (fullscreen):
            flags |= pygame.FULLSCREEN
        pygame.display.set_mode([width, height], flags)
        self.surface = pygame.display.get_surface()
        self.maxy = height - 1
        
    def erase(self):
        if (self.surface):
            self.surface.fill((0,0,0))
        
    def flip(self):
        self.frame_num = self.frame_num + 1
        pygame.display.flip()