"""
The standard framework for an editor that uses the MotherHamster Engine
...Has a very boring name right now...
(C) Copyright 2004 Hamster Republic Productions
"""

########################################################################
from motherhamster.hamsterwheel import HamsterWheel
from motherhamster import cheekpouches
from motherhamster import asseteditor
from motherhamster.debugimagewindow import DebugImageBrowser

import threading
import wx
import profilingwindows
import platform

########################################################################
def _friendly_class_name(class_object):
    return repr(class_object).strip("<'>").split('.')[-1].split(' at ')[0]

########################################################################
class NewAssetWindow(wx.Frame):
    """ A window that you can use to pick a folder and name
        to create in the data folder"""
        
    def __init__(self, parent, cheekpouch):
        wx.Frame.__init__(self, parent, -1, 'NewAsset', size = (200, 400),
                          style=wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

        self.cheekpouch = cheekpouch
        
        self.panel = wx.Panel(self, -1)

        self.AssetDirectoryTree = wx.TreeCtrl(self.panel, -1, style = wx.TR_HAS_BUTTONS)
        item_size = (16,16)
        self.image_list = wx.ImageList(item_size[0], item_size[1])
        self.folder_id = self.image_list.Add(wx.ArtProvider_GetBitmap(wx.ART_FOLDER,      wx.ART_OTHER, item_size))
        self.openfolder_id = self.image_list.Add(wx.ArtProvider_GetBitmap(wx.ART_FILE_OPEN,   wx.ART_OTHER, item_size))
        self.assetimage_id = self.image_list.Add(wx.ArtProvider_GetBitmap(wx.ART_NORMAL_FILE, wx.ART_OTHER, item_size))
        self.AssetDirectoryTree.SetImageList(self.image_list)

        self.root = self.AssetDirectoryTree.AddRoot("")
        self.AssetDirectoryTree.SetPyData(self.root, ".")
        self.tree_guide = {}
        
        data_dish = cheekpouches.GetFoodDish("data")
        dir_list = data_dish.FindSubdirectoryPaths()
        for dir in dir_list:
            self.AddFoldersForPath(dir)
        asset_list = data_dish.FindFilesByExtension(cheekpouch.AssetExtensions())
        for asset_path in asset_list:
            self.AddExistingAssetByPath(asset_path)
        self.AssetDirectoryTree.Expand(self.root)
        
        self.AssetName = wx.TextCtrl(self.panel)
        self.Bind(wx.EVT_TEXT, self.OnEdit, self.AssetName)

        self.CreateButton = wx.Button(self.panel, -1, "Create")
        self.CreateButton.Enable(False)
        self.CreateButton.SetDefault()
        self.Bind(wx.EVT_BUTTON, self.CreateAsset, self.CreateButton)

        self.CancelButton = wx.Button(self.panel, -1, "Cancel")
        self.CancelButton.Enable(True)
        self.Bind(wx.EVT_BUTTON, self.OnCloseWindow, self.CancelButton)
        
        buttonbox = wx.BoxSizer(wx.HORIZONTAL)
        buttonbox.Add((0,0), 1)
        buttonbox.Add(self.CreateButton, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        buttonbox.Add((0,0), 1)
        buttonbox.Add(self.CancelButton, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        buttonbox.Add((0,0), 1)
        
        verticalbox = wx.BoxSizer(wx.VERTICAL)
        label = wx.StaticText(self.panel, label = "pick a directory:")
        verticalbox.Add(label, 0, wx.ALIGN_LEFT, border = 5)
        verticalbox.Add(self.AssetDirectoryTree, 1, wx.EXPAND, border = 5)
        label = wx.StaticText(self.panel, label = "choose an asset name:")
        verticalbox.Add(label, 0, wx.ALIGN_LEFT, border = 5)
        verticalbox.Add(self.AssetName, 0, wx.EXPAND, border = 5)
        verticalbox.Add(buttonbox, 0, wx.EXPAND)
        
        self.panel.SetSizerAndFit(verticalbox)
        self.SetSize(self.GetBestSize())

    def AddFoldersForPath(self, asset_dir):
        parent_node = self.root
        path_list = asset_dir.split('/')
        path = ""
        for path_element in path_list:
            path += path_element
            if (self.tree_guide.has_key(path)):
                parent_node = self.tree_guide[path]
            else:
                new_node = self.AssetDirectoryTree.AppendItem(parent_node, path_element)
                self.AssetDirectoryTree.SetItemImage(new_node, self.folder_id, wx.TreeItemIcon_Normal)
                self.AssetDirectoryTree.SetItemImage(new_node, self.openfolder_id, wx.TreeItemIcon_Expanded)
                self.tree_guide[path] = new_node
                self.AssetDirectoryTree.SetPyData(new_node, path)
                parent_node = new_node
            path += '/'
        return parent_node
    
    def AddExistingAssetByPath(self, asset_path):
        parent_node = self.AddFoldersForPath(asset_path.path)
        if (not self.tree_guide.has_key(asset_path)):
            new_node = self.AssetDirectoryTree.AppendItem(parent_node, asset_path.assetid)
            self.AssetDirectoryTree.SetItemImage(new_node, self.assetimage_id, wx.TreeItemIcon_Normal)
            self.AssetDirectoryTree.SetPyData(new_node, asset_path.path)
            self.tree_guide[asset_path] = new_node

    def IsValidAssetName(self, value):
        for char in "?/\\|:<>\"":
            if char in value:
                return False
        #CHECK: prevent duplicate names
        return value != ""
    
    def OnEdit(self, event):
        value = self.AssetName.GetValue()
        self.CreateButton.Enable(self.IsValidAssetName(value))

    def CreateAsset(self, event):
        path = self.AssetDirectoryTree.GetPyData(self.AssetDirectoryTree.GetSelection())
        assetid = self.AssetName.GetValue()
        data_dish = cheekpouches.GetFoodDish("data")
        asset_path = data_dish.GetAssetPath(path, assetid, self.cheekpouch.NewAssetExtension())
        self.cheekpouch.CreateNewAsset(asset_path)
        self.Destroy()
        self.GetParent().RefreshDirectoryList()
        
    def OnCloseWindow(self, event):
        self.Destroy()
    
########################################################################
class AssetBrowser(wx.Frame):
    """ A window that lets you browse the different asset types
        """
    _Browser = None
    def __Activate():
        if (AssetBrowser._Browser == None):
            AssetBrowser._Browser = AssetBrowser(None)
            AssetBrowser._Browser.Show()
        else:
            AssetBrowser._Browser.Raise()
    Activate = staticmethod(__Activate)
    
    def __init__(self, parent):
        wx.Frame.__init__(self, parent, -1, 'AssetBrowser', size = (200, 400),
                          style=wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)
        
        classlist = cheekpouches.AvailableAssetClassTypes()
        self._choicelist = {}
        self._class_type = None
        for class_type in classlist:
            clean_long_name = repr(class_type).strip("<'>").split('.')[-1]
            clean_short_name = clean_long_name.split(' at ')[0]
            if (self._choicelist.has_key(clean_short_name)):
                collided_obj = self._choicelist[clean_short_name];
                collided_long_name = repr(collided_obj).strip("<'>").split('.')[-1]
                self._choicelist[clean_long_name] = class_type
                self._choicelist[collided_long_name] = collided_obj
                del self._choicelist[clean_short_name]
            else:
                self._choicelist[clean_short_name] = class_type

        self.panel = wx.Panel(self, -1)
        
        self.AssetTypeChoice = wx.Choice(self.panel, -1, choices = self._choicelist.keys())
        self.Bind(wx.EVT_CHOICE, self.OnChoice, self.AssetTypeChoice)

        self.AssetDirectoryList = wx.TreeCtrl(self.panel, -1, style = wx.TR_HAS_BUTTONS)
        item_size = (16,16)
        self.image_list = wx.ImageList(item_size[0], item_size[1])
        self.folder_id = self.image_list.Add(wx.ArtProvider_GetBitmap(wx.ART_FOLDER,      wx.ART_OTHER, item_size))
        self.openfolder_id = self.image_list.Add(wx.ArtProvider_GetBitmap(wx.ART_FILE_OPEN,   wx.ART_OTHER, item_size))
        self.assetimage_id = self.image_list.Add(wx.ArtProvider_GetBitmap(wx.ART_NORMAL_FILE, wx.ART_OTHER, item_size))
        self.AssetDirectoryList.SetImageList(self.image_list)
        
        self.AssetDirectoryList.Bind(wx.EVT_LEFT_DCLICK, self.EditSelectedAsset)
        self.Bind(wx.EVT_TREE_ITEM_COLLAPSING, self.KeepRootOpen, self.AssetDirectoryList)
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.SetEditEnabled, self.AssetDirectoryList)

        self.EditButton = wx.Button(self.panel, -1, "Edit")
        self.EditButton.Enable(False)
        self.EditButton.SetDefault()
        self.Bind(wx.EVT_BUTTON, self.EditSelectedAsset, self.EditButton)

        self.NewButton = wx.Button(self.panel, -1, "New")
        self.NewButton.Enable(False)
        self.Bind(wx.EVT_BUTTON, self.NewAsset, self.NewButton)
        
        buttonbox = wx.BoxSizer(wx.HORIZONTAL)
        buttonbox.Add((0,0), 1)
        buttonbox.Add(self.EditButton, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        buttonbox.Add((0,0), 1)
        buttonbox.Add(self.NewButton, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        buttonbox.Add((0,0), 1)
        
        verticalbox = wx.BoxSizer(wx.VERTICAL)
        verticalbox.Add(self.AssetTypeChoice, 0, wx.EXPAND)
        verticalbox.Add(self.AssetDirectoryList, 1, wx.EXPAND)
        verticalbox.SetItemMinSize(self.AssetDirectoryList, 100, 200)
        verticalbox.Add(buttonbox, 0, wx.EXPAND)
        
        self.panel.SetSizerAndFit(verticalbox)
        self.SetSize(self.GetBestSize())
        
    #---------------------------------------------
    def OnChoice(self, event):
        self.NewButton.Enable(True)
        self.RefreshDirectoryList()

    #---------------------------------------------
    def RefreshDirectoryList(self):
        class_name = self.AssetTypeChoice.GetStringSelection()
        self._class_type = self._choicelist[class_name]

        self.AssetDirectoryList.DeleteAllItems()       
        self.root = self.AssetDirectoryList.AddRoot(class_name)
        self.AssetDirectoryList.SetItemImage(self.root, self.folder_id, wx.TreeItemIcon_Normal)
        self.AssetDirectoryList.SetItemImage(self.root, self.openfolder_id, wx.TreeItemIcon_Expanded)
        self.tree_guide = {}
        
        asset_list = cheekpouches.AvailableAssetIDs(self._class_type, rescan_list = True)
        for asset_path in asset_list:
            self.AddTreeItemByPath(asset_path)

        self.AssetDirectoryList.Expand(self.root)
        (root_item, cookie) = self.AssetDirectoryList.GetFirstChild(self.root)
        while root_item.IsOk():
            self.AssetDirectoryList.Expand(root_item)
            (root_item, cookie) = self.AssetDirectoryList.GetNextChild(self.root, cookie)
            
        best_size = self.GetBestSize()
        current_size = self.GetSize()
        new_size = (max(best_size[0], current_size[0]), max(best_size[1], current_size[1]))
        self.SetSize(new_size)
                            
    #---------------------------------------------
    def AddTreeItemByPath(self, asset_path):
        node = self.root
        path_list = [asset_path.root]
        path_list.extend(asset_path.split('/'))
        path = ""
        for path_element in path_list:
            path += '/' + path_element
            if (self.tree_guide.has_key(path)):
                node = self.tree_guide[path]
            else:
                new_node = self.AssetDirectoryList.AppendItem(node, path_element)
                self.AssetDirectoryList.SetItemImage(new_node, self.folder_id, wx.TreeItemIcon_Normal)
                self.AssetDirectoryList.SetItemImage(new_node, self.openfolder_id, wx.TreeItemIcon_Expanded)
                self.tree_guide[path] = new_node
                node = new_node
        self.AssetDirectoryList.SetPyData(node, asset_path)
        self.AssetDirectoryList.SetItemImage(node, self.assetimage_id, wx.TreeItemIcon_Normal)

    #---------------------------------------------
    def SetEditEnabled(self, event):
        item = event.GetItem()
        self.EditButton.Enable(self.AssetDirectoryList.GetPyData(item) != None)

    #---------------------------------------------
    def EditSelectedAsset(self, event):
        asset_path = self.AssetDirectoryList.GetPyData(self.AssetDirectoryList.GetSelection())
        if (asset_path != None):
            asset = cheekpouches.Disgorge(self._class_type, asset_path, True)
            asseteditor.AssetEditor.Edit(asset)

    #---------------------------------------------
    def NewAsset(self, event):
        new_window = NewAssetWindow(self, cheekpouches.GetPouch(self._class_type))
        new_window.Show(True)
    
    #---------------------------------------------
    def KeepRootOpen(self, event):
        item = event.GetItem()
        if (item == self.root or self.AssetDirectoryList.GetItemParent(item) == self.root):
            event.Veto()
        
    #---------------------------------------------
    def OnCloseWindow(self, event):
        self.Destroy()
        if (AssetBrowser._Browser == self):
            AssetBrowser._Browser = None
        
########################################################################
class _EditorWindow(wx.Frame):
    """ A wxWindows/wxWidgets app that is designed to run parallel to a
        HamsterWheel based app (meaning a wxApp that runs alongside a
        pyGame window)
        It is designed to run either as a seperate thread from or 
        in the same thread as the HamsterWheel app, depending on what the 
        current platform supports
        """
    def __init__(self, lock=None):
        wx.Frame.__init__(self, None, -1, 'EditorWindow', size = (350, 300),
                          style=wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE)

        self.object_lock = lock
        
        # CHECK: should set an icon?
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

        self.CreateStatusBar(1, wx.ST_SIZEGRIP)

        # Make a File menu
        self.mainmenu = wx.MenuBar()
        menu = wx.Menu()
        
        browseID = wx.NewId()
        item = menu.Append(browseID, '&Browse Assets\tAlt-B', 'Look at available assets')
        self.Bind(wx.EVT_MENU, self.OnAssetBrowser, item)

        profileID = wx.NewId()
        item = menu.Append(profileID, '&Profiler\tAlt-P', 'Bring up the profiling window')
        self.Bind(wx.EVT_MENU, self.OnProfiler, item)

        debugimageID = wx.NewId()
        item = menu.Append(debugimageID, 'Debug &Images\tAlt-I', 'Browse debug images')
        self.Bind(wx.EVT_MENU, self.OnDebugImages, item)

        debugdrawID = wx.NewId()
        item = menu.Append(debugdrawID, 'Debug &Draw\tAlt-D', 'Toggle Debug Draw', wx.ITEM_CHECK)
        item.Check(HamsterWheel.the_game.draw_debug_enabled)
        self.Bind(wx.EVT_MENU, self.OnToggleDebugDraw, item)

        exitID = wx.NewId()       
        item = menu.Append(exitID, 'E&xit\tAlt-X', 'Get the heck outta here!')
        self.Bind(wx.EVT_MENU, self.OnFileExit, item)
        wx.App_SetMacExitMenuItemId(exitID)
        
        self.mainmenu.Append(menu, '&Tools')

        aTable = wx.AcceleratorTable([(wx.ACCEL_ALT,  ord('B'), browseID),
                                      (wx.ACCEL_ALT,  ord('P'), profileID),
                                      (wx.ACCEL_ALT,  ord('I'), debugimageID),
                                      (wx.ACCEL_ALT,  ord('X'), exitID)])
        self.SetAcceleratorTable(aTable)
        
        self.SetMenuBar(self.mainmenu)
        
        self.panel = wx.Panel(self, -1)

        description = wx.StaticText(self.panel, label = "game speed %:")
        self.speedslider = wx.Slider(self.panel, value=HamsterWheel.the_game.game_clock_speed*100, minValue=0, maxValue=100, pos=(0,0), style=wx.SL_HORIZONTAL | wx.SL_AUTOTICKS)
        self.speedslider.Bind(wx.EVT_SCROLL_THUMBTRACK, self.OnSliderChange)
        self.speedslider.Bind(wx.EVT_SCROLL_THUMBRELEASE, self.OnSliderChange)
        self.speedslider.Bind(wx.EVT_SCROLL_ENDSCROLL, self.OnSliderChange)
        self.speedslider.SetTickFreq(10, 1)
                
        self.log = wx.TextCtrl(self.panel, -1, size=(200, 100),
                              style = wx.TE_MULTILINE|wx.TE_READONLY|wx.VSCROLL)
        wx.Log_SetActiveTarget(wx.LogTextCtrl(self.log))

        verticalbox = wx.BoxSizer(wx.VERTICAL)
        speedbox = wx.BoxSizer(wx.HORIZONTAL)
        speedbox.Add(description, 0, border = 5, flag = wx.FIXED_MINSIZE | wx.ALL)
        speedbox.Add(self.speedslider, 1, border = 5, flag = wx.EXPAND | wx.ALL | wx.ALIGN_LEFT)
        verticalbox.Add(speedbox, 0, wx.EXPAND)
        verticalbox.Add(self.log, 1, wx.EXPAND)
        speedbox.SetItemMinSize(self.log, 200, 60)
        self.panel.SetSizerAndFit(verticalbox)
        self.SetMinSize(verticalbox.GetMinSize())
        self.panel.SetClientSize(verticalbox.GetSize())
        self.SetClientSize(self.panel.GetSize())
                
    #---------------------------------------------
    # Menu methods
    def OnFileExit(self, *event):
        self.Close()

    #---------------------------------------------
    def OnCloseWindow(self, event):
        HamsterWheel.the_game.game_clock_speed = 1.0
        global _editor
        _editor.DropApp()
        self.Destroy()

    #---------------------------------------------
    def OnSliderChange(self, event):
        percent = self.speedslider.GetValue()
        HamsterWheel.the_game.game_clock_speed = percent/100.0;
    
    #---------------------------------------------
    def OnAssetBrowser(self, event):
        browser = AssetBrowser(self)
        browser.Show(True)

    #---------------------------------------------
    def OnProfiler(self, event):
        profiler_window = profilingwindows.TimingGraph()
        profiler_window.Show(True)

    #---------------------------------------------
    def OnDebugImages(self, event):
        DebugImageBrowser.Activate()

    #---------------------------------------------
    def OnToggleDebugDraw(self, event):
        HamsterWheel.the_game.draw_debug_enabled = not HamsterWheel.the_game.draw_debug_enabled
        
########################################################################
class _EditorApp(wx.App):
    def __init__(self, lock=None, redirect=False, filename=None, useBestVisual=False):
        self.lock = lock
        self.Window = None
        wx.App.__init__(self, redirect, filename, useBestVisual)
        
    def OnInit(self):
        self.Window = _EditorWindow(self.lock)
        self.Window.Show()
        return True
    
    def ActivateWindow(self):
        self.Window.Raise()
        
    def CloseWindow(self):
        self.Window.Close()

########################################################################
class EditorBase(object):        
    """ The EditorBase class just does some generic stuff needed for the core
        functionality of the editor """
    def __init__(self):
        HamsterWheel.the_game.register_for_events(self)
        self._editor_app = None

    def Kill(self):
        pass
    
    def Activate(self):
        pass

    def hamsterwheel_event(self, eventtype):
        pass
        
########################################################################
class EditorThreaded(EditorBase):
    """ The EditorThreaded class implements a level editor whose windows run in
        another thread. It expects it's public interface to be called from the
        main thread (the one that created the "EditorThreaded") """
        
    def __init__(self):
        self.__editor_lock = threading.Lock()
        self.__editor_thread = None
        EditorBase.__init__(self)

    def __thread_start(self):
        self._editor_app = _EditorApp(self.__editor_lock, False)
        self._editor_app.MainLoop()

    def Activate(self):
        if (self._editor_app != None):
            self._editor_app.ActivateWindow()
        if (self._editor_app == None or self.__editor_thread == None or not self.__editor_thread.IsActive()):
            self.__editor_thread = threading.Thread(name="EditorThread", target=self.__thread_start)
            self.__editor_thread.start()
                
    def Kill(self):
        if (self._editor_app):
            self._editor_app.CloseWindow()
            self.DropApp()
        self.__editor_thread = None

    def DropApp(self):
        self._editor_app = None
    
    def IsActive(self):
        return self._editor_app != None
    
    def hamsterwheel_event(self, eventtype):
        if (eventtype == HamsterWheel.events.UpdateStart or eventtype == HamsterWheel.events.DrawStart):
            self.__editor_lock.acquire()
        elif (eventtype == HamsterWheel.events.UpdateEnd or eventtype == HamsterWheel.events.DrawEnd):
            self.__editor_lock.release()
                
########################################################################
class EditorSimple(EditorBase):
    """ The EditorSimple class implements a level editor who runs in the main
        thread. It makes it's windows message loops run inbetween frames """
    def __init__(self):
        EditorBase.__init__(self)
        self.__need_drop_app = False
    
    def Activate(self):
        if (self._editor_app == None):
            self._editor_app = _EditorApp(None, False)
        else:
            self._editor_app.ActivateWindow()
            
    def Kill(self):    
        if (self._editor_app != None):
            self.__need_drop_app = True
            self._editor_app.CloseWindow()
    
    def DropApp(self):
        self.__need_drop_app = True

    def IsActive(self):
        return self._editor_app != None
    
    def hamsterwheel_event(self, eventtype):
        if (eventtype == HamsterWheel.events.FrameEnd and self._editor_app != None):
            self._editor_app.Yield(True)
            if (self.__need_drop_app):
                self._editor_app = None
                self.__need_drop_app = False

########################################################################
def SpawnEditor():
    global _editor

    if (_editor == None):
        # decide which editor to use
        platform_string = platform.platform(aliased = True, terse = True)
        __use_threading = 'Windows' in platform_string
        if __use_threading:
            print "Using Threaded Editor"
            _editor = EditorThreaded()
        else:
            _editor = EditorSimple()
        
    _editor.Activate()
        
########################################################################
def KillEditor():
    global _editor
    _editor.Kill()

########################################################################
_editor = None

if __name__ == '__main__':
    class FakeGame(object):
        def __init__(self):
            self.game_clock_speed = 1.0
            self.draw_debug_enabled = True
            
        def register_for_events(self, obj):
            pass        
    HamsterWheel.the_game = FakeGame()

    SpawnEditor()
    while _editor.IsActive():
        _editor.hamsterwheel_event(HamsterWheel.events.FrameStart)
        _editor.hamsterwheel_event(HamsterWheel.events.UpdateStart)
        _editor.hamsterwheel_event(HamsterWheel.events.UpdateEnd)
        _editor.hamsterwheel_event(HamsterWheel.events.DrawStart)
        _editor.hamsterwheel_event(HamsterWheel.events.DrawEnd)
        _editor.hamsterwheel_event(HamsterWheel.events.FrameEnd)
