"""
Tools and stuff for doing ad-hoc profiling
(C) Copyright 2005 Hamster Republic Productions
"""

########################################################################
import wx
import sys
import types
import weakref
from motherhamster import profiler
from wx.lib.mixins.listctrl import CheckListCtrlMixin
from wx.lib.mixins.listctrl import ColumnSorterMixin
from wx.lib.mixins.listctrl import ListCtrlAutoWidthMixin

########################################################################
GraphColors = [
        "RED",
        "GREEN",
        "BLUE",
        "ORANGE",
        "SKY BLUE",
        "MAGENTA",
        "SIENNA",
        "SPRING GREEN",
        "SLATE BLUE",
        "GOLD",
        "VIOLET RED",
        "DARK VIOLET",
        "DARK RED",
        "DARK GREEN",
        "DARK BLUE"
        ]

########################################################################
class ModuleBrowser(wx.Frame):
    """ A window that lets you browse the different asset types
        """
    def __init__(self, parent = None):
        wx.Frame.__init__(self, parent, -1, 'ModuleBrowser', size = (200, 400),
                          style=wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

        self.panel = wx.Panel(self, -1)
                
        self.module_list = sys.modules.keys()
        self.module_list.sort()

        self.module_tree = wx.TreeCtrl(self.panel, -1, wx.DefaultPosition, wx.DefaultSize)
        moduleroot = self.module_tree.AddRoot("modules")

        motherhamsterroot = self.module_tree.AppendItem(moduleroot, "motherhamster")
        builtinroot = self.module_tree.AppendItem(moduleroot, "built-in")
        sitepackagesroot = self.module_tree.AppendItem(moduleroot, "site-packages")
        
        for modulename in self.module_list:
            module = sys.modules[modulename]
            if (isinstance(module, types.NoneType)):
                pass
                #print "skipping", modulename, module, "cause it's NoneType"
            else:
                parent_item = moduleroot
                module_string = repr(module).replace('\\', '/').lower()
                if modulename == '__main__':
                    pass
                elif modulename.startswith('motherhamster.'):
                    parent_item = motherhamsterroot
                    modulename = modulename[14:]
                elif '/site-packages/' in module_string:
                    parent_item = sitepackagesroot
                elif '(built-in)' in module_string or '/lib/' in module_string or '/dlls/' in module_string:
                    parent_item = builtinroot
                module_item = self.module_tree.AppendItem(parent_item, modulename)
                self.module_tree.SetPyData(module_item, module)
                self.module_tree.SetItemHasChildren(module_item, True)

        self.module_tree.Expand(moduleroot)
            
        self.Bind(wx.EVT_TREE_ITEM_EXPANDING, self.OnItemExpanding, self.module_tree)
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.OnSelChanged, self.module_tree)
        
        self.RegisterButton = wx.Button(self.panel, -1, "Register")
        self.RegisterButton.Enable(False)
        self.Bind(wx.EVT_BUTTON, self.RegisterFunction, self.RegisterButton)
        
        buttonbox = wx.BoxSizer(wx.HORIZONTAL)
        buttonbox.Add((0,0), 1)
        buttonbox.Add(self.RegisterButton, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        buttonbox.Add((0,0), 1)
        
        verticalbox = wx.BoxSizer(wx.VERTICAL)
        verticalbox.Add(self.module_tree, 1, wx.EXPAND)
        verticalbox.SetItemMinSize(self.module_tree, 200, 500)
        verticalbox.Add(buttonbox, 0, wx.EXPAND)
        
        self.panel.SetSizerAndFit(verticalbox)
        self.SetSize(self.GetBestSize())

    def _ShouldObjectBeShown(self, name, item_obj):
        if (name == '__class__'):
            return False
        if (name == 'im_class'):
            return False
        if (isinstance(item_obj, types.NoneType)):
            return False
        if (isinstance(item_obj, types.ClassType)):
            return True
        if (isinstance(item_obj, types.TypeType)):
            return True
        if (isinstance(item_obj, types.MethodType)):
            return True
        if (isinstance(item_obj, types.FunctionType)):
            return True
        if (isinstance(item_obj, types.BuiltinFunctionType)):
            return True
        if (isinstance(item_obj, types.BuiltinMethodType)):
            return True
        return False
    
    def _ShouldHaveChildren(self, item_obj):
        if isinstance(item_obj, types.BuiltinFunctionType) or isinstance(item_obj, types.BuiltinMethodType):
            return False
        for key, child_object in item_obj.__dict__.items():
            if self._ShouldObjectBeShown(key, child_object):
                return True
        return False
    
    def _EnumerateChildren(self, parent):
        item_obj = self.module_tree.GetPyData(parent)
        print "enumerating: ", item_obj, type(item_obj)
        print dir(item_obj)
        keys = item_obj.__dict__.keys()
        keys.sort()
        for key in keys:
            child_object = item_obj.__dict__[key]
            if (self._ShouldObjectBeShown(key, child_object)):
                child = self.module_tree.AppendItem(parent, key)
                self.module_tree.SetPyData(child, child_object)
                self.module_tree.SetItemHasChildren(child, self._ShouldHaveChildren(child_object))
            else:
                print 'skipping:',key, type(child_object)
        self.module_tree.SetItemHasChildren(parent, self.module_tree.GetChildrenCount(parent) > 0)
        
    def _GetPathToObjectForTreeItem(self, item):
        parent_item = self.module_tree.GetItemParent(item)
        parent = self.module_tree.GetPyData(parent_item)
        name = self.module_tree.GetItemText(item)
        return parent, name
        
    def OnSelChanged(self, event):
        item = event.GetItem()
        item_obj = self.module_tree.GetPyData(item)
        print item_obj, type(item_obj)
        print dir(item_obj)
        if isinstance(item_obj, types.FunctionType) \
                or isinstance(item_obj, types.MethodType) \
                or isinstance(item_obj, types.BuiltinFunctionType) \
                or isinstance(item_obj, types.BuiltinMethodType):
            parent, key = self._GetPathToObjectForTreeItem(item)
            self.RegisterButton.Enable(not profiler.IsFunctionRegistered(parent, key))
        else:
            print "unregisterable:", type(item_obj)
            self.RegisterButton.Enable(False)
        
    def OnItemExpanding(self, event):
        item = event.GetItem()
        print "children:",self.module_tree.ItemHasChildren(item),"count:",self.module_tree.GetChildrenCount(item)
        if (self.module_tree.ItemHasChildren(item) 
                and self.module_tree.GetChildrenCount(item) == 0):
            self._EnumerateChildren(item)
        event.Skip()
    
    def RegisterFunction(self, event):
        selected = self.module_tree.GetSelection()
        parent, key = self._GetPathToObjectForTreeItem(selected)
        parent_item = self.module_tree.GetItemParent(selected)
        parent_name = self.module_tree.GetItemText(parent_item)
        profiler.RegisterFunction(parent, key, parent_name)
        self.module_tree.Delete(selected)
        self.RegisterButton.Enable(False)
    
    #---------------------------------------------
    def OnCloseWindow(self, event):
        self.Destroy()

########################################################################
class ReusablePointList(list):
    """ a list class that is always filled with points and can have it's length set """
    def __init__(self):
        list.__init__(self)
        self.usable_len = 0
        
    def __len__(self):
        return self.usable_len
    
    def set_len(self, length):
        while list.__len__(self) < length:
            self.append(wx.Point(0,0))
            print 'added',list.__len__(self)
        self.usable_len = length

########################################################################
class PerformanceGraphPanel(wx.Panel):        
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)
        self.SetBackgroundColour(wx.WHITE)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.prev_index = 0
        self.fps_pen = wx.Pen("BLACK", 1)
        self.end_pen = wx.Pen("YELLOW", 1)
        self.eraser = wx.Brush("WHITE")
        self.no_pen = wx.Pen("WHITE", 0)

        self.pens = map(lambda(x) : wx.Pen(x, 1), GraphColors)
        self.y_max = 1/12.0

    def OnSize(self,evt):
        self.Refresh()
        
    def OnPaint(self, evt):
        dc = wx.PaintDC(self)
        dc.Clear()
        dc.BeginDrawing()
        self.DrawPerfLines(dc, 0, profiler.FunctionTracker.SampleCount)
        dc.EndDrawing()            
        self.prev_index = profiler.FunctionTracker.Index

    def DrawPerfLines(self, dc, start_index, end_index):
        size = self.GetSize()
        xscale = 1.0*size[0]/profiler.FunctionTracker.SampleCount
        yscale = size[1]/self.y_max
        pen_index = 0
        for profile in profiler.FunctionProfiles:
            if (profile.is_active()):
                dc.SetPen(self.pens[pen_index % len(GraphColors)])
                y_point = size[1] - yscale*profile.owned(start_index)
                x_point = start_index*xscale
                line_list = []
                for i in xrange(start_index + 1, end_index):
                    new_y = size[1] - yscale*profile.owned(i)
                    new_x = i*xscale
                    line_list.append((x_point, y_point, new_x, new_y))
                    x_point = new_x
                    y_point = new_y
                dc.DrawLineList(line_list)
            pen_index += 1
        dc.SetPen(self.fps_pen)
        y_point = size[1] - yscale*profiler.FrameTime(start_index)
        x_point = start_index*xscale
        for i in xrange(start_index + 1, end_index):
            new_y = size[1] - yscale*profiler.FrameTime(i)
            new_x = i*xscale
            dc.DrawLine(x_point, y_point, new_x, new_y)
            x_point = new_x
            y_point = new_y

    def UpdateGraph(self):
        new_index = profiler.FunctionTracker.Index
        if new_index != self.prev_index:
            size = self.GetSize()
            dc = wx.ClientDC(self)
            dc.BeginDrawing()
            prev_x = self.prev_index*size[0]/profiler.FunctionTracker.SampleCount + 1
            last_x = new_index*size[0]/profiler.FunctionTracker.SampleCount
            dc.SetPen(self.no_pen)
            dc.SetBrush(self.eraser)
            if new_index < self.prev_index:
                dc.DrawRectangle(prev_x, 0, size[0] - prev_x , size[1])
                dc.DrawRectangle(0, 0, last_x, size[1] + 2)
                self.DrawPerfLines(dc, self.prev_index - 1, profiler.FunctionTracker.SampleCount)
                self.DrawPerfLines(dc, 0, new_index)
            else:
                dc.DrawRectangle(prev_x, 0, max(1, last_x - prev_x) + 2, size[1])
                self.DrawPerfLines(dc, self.prev_index - 1, new_index)
            dc.SetPen(self.end_pen)
            dc.DrawLine(last_x + 1, 0, last_x + 1, size[1])
            dc.EndDrawing()
            self.prev_index = new_index
        
########################################################################
class ProfiledFunctionList(wx.ListCtrl, CheckListCtrlMixin, ListCtrlAutoWidthMixin):
    def __init__(self, parent):
        wx.ListCtrl.__init__(
            self, parent, -1, 
            style=wx.LC_REPORT|wx.LC_HRULES|wx.LC_VRULES|wx.LC_SORT_ASCENDING
            )
        CheckListCtrlMixin.__init__(self)
        ListCtrlAutoWidthMixin.__init__(self)

        self.InsertColumn(0, "Object")
        self.InsertColumn(1, "Function")
        self.InsertColumn(2, "calls")
        self.InsertColumn(3, "time")
        self.InsertColumn(4, "owned")
        self.InsertColumn(5, "frame %")
        self.InsertColumn(6, "frame max")

    def OnCheckItem(self, item_index, flag):
        self.GetParent().CheckItem(item_index, flag)

########################################################################
class SortableFunctionListPanel(wx.Panel, ColumnSorterMixin):
    """ A panel that can handle sorting of a list ctrl and being sized """
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1, style=wx.WANTS_CHARS)

        self.perf_graph = PerformanceGraphPanel(self)
        
        self.frame_average = 30
        self.function_list = ProfiledFunctionList(self)

        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.OnItemSelected, self.function_list)
        self.Bind(wx.EVT_LIST_ITEM_DESELECTED, self.OnItemDeselected, self.function_list)
                
        fpslabel = wx.StaticText(self, label = "fps:")
        self.fps_display = wx.TextCtrl(self, -1)
        self.fps_display.SetClientSize(self.fps_display.GetTextExtent("000.0"))
        self.fps_display.SetMinSize(self.fps_display.GetSize())

        framelabel = wx.StaticText(self, label = "frame time:")
        self.frame_time = wx.TextCtrl(self, -1)
        self.frame_time.SetClientSize(self.frame_time.GetTextExtent("0000.0 ms"))
        self.frame_time.SetMinSize(self.frame_time.GetSize())

        averagelabel = wx.StaticText(self, label = "avg. frames:")
        self.frame_average_ctrl = wx.SpinCtrl(self, -1, "label:")
        self.frame_average_ctrl.SetRange(1, profiler.FunctionTracker.SampleCount)
        self.frame_average_ctrl.SetValue(self.frame_average)
        desired_size = (self.frame_time.GetSize()[0], self.frame_average_ctrl.GetSize()[1])
        self.frame_average_ctrl.SetMinSize(desired_size)

        refreshlabel = wx.StaticText(self, label = "refresh ms:")
        self.refresh_ctrl = wx.SpinCtrl(self, -1, "label:")
        self.refresh_ctrl.SetRange(1, 10000)
        self.refresh_ctrl.SetValue(800)
        desired_size = (self.frame_time.GetSize()[0], self.refresh_ctrl.GetSize()[1])
        self.refresh_ctrl.SetMinSize(desired_size)

        self.UpdateData()
        ColumnSorterMixin.__init__(self, self.function_list.GetColumnCount())

        item_size = (16,16)
        image_list = self.function_list.GetImageList(wx.IMAGE_LIST_SMALL)
        self.sm_up = image_list.Add(wx.ArtProvider_GetBitmap(wx.ART_GO_UP, wx.ART_TOOLBAR, item_size))
        self.sm_dn = image_list.Add(wx.ArtProvider_GetBitmap(wx.ART_GO_DOWN, wx.ART_TOOLBAR, item_size))
        
        framebox = wx.BoxSizer(wx.HORIZONTAL)
        framebox.Add(refreshlabel, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        framebox.Add(self.refresh_ctrl, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        framebox.Add((0,0), 1)
        framebox.Add(averagelabel, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        framebox.Add(self.frame_average_ctrl, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        framebox.Add((0,0), 2)
        framebox.Add(fpslabel, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        framebox.Add(self.fps_display, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        framebox.Add((0,0), 1)
        framebox.Add(framelabel, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        framebox.Add(self.frame_time, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        
        self.AddButton = wx.Button(self, -1, "Add Function")
        self.AddButton.Enable(True)
        self.Bind(wx.EVT_BUTTON, self.ShowFunctionBrowser, self.AddButton)

        self.RemoveButton = wx.Button(self, -1, "Remove Function")
        self.RemoveButton.Enable(False)
        self.Bind(wx.EVT_BUTTON, self.RemoveFunction, self.RemoveButton)
        
        buttonbox = wx.BoxSizer(wx.HORIZONTAL)
        buttonbox.Add((0,0), 1)
        buttonbox.Add(self.AddButton, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        buttonbox.Add((0,0), 1)
        buttonbox.Add(self.RemoveButton, 0, wx.ALIGN_CENTER | wx.ALL, border = 5)
        buttonbox.Add((0,0), 1)

        verticalbox = wx.BoxSizer(wx.VERTICAL)
        verticalbox.Add(self.perf_graph, 1, wx.EXPAND)
        verticalbox.Add(framebox, 0, wx.EXPAND)
        verticalbox.Add(self.GetListCtrl(), 1, wx.EXPAND)
        verticalbox.SetItemMinSize(self.GetListCtrl(), 460, 200)
        verticalbox.Add(buttonbox, 0, wx.EXPAND)

        self.SetSizerAndFit(verticalbox)
        
        self.Bind(wx.EVT_TIMER, self.UpdateTimer)
        self.UpdateStatTimer = wx.Timer(self)
        self.UpdateStatTimer.Start(self.refresh_ctrl.GetValue())               

    def GetListCtrl(self):
        return self.function_list
    
    def GetSortImages(self):
        return (self.sm_dn, self.sm_up)
    
    def InitializeData(self):
        self.itemDataMap = {}
        self.profile_ref_map = {}
        self.function_list.DeleteAllItems()
        item_key = 0
        color_index = 0
        for profile in profiler.FunctionProfiles:
            item_index = self.function_list.InsertStringItem(sys.maxint, profile.object_name)
            
            item = self.function_list.GetItem(item_index)
            color = GraphColors[color_index % len(GraphColors)]
            item.SetTextColour(GraphColors[color_index % len(GraphColors)])
            self.function_list.SetItem(item)
            color_index += 1
            
            self.function_list.SetStringItem(item_index, 1, profile.function_name)
            self.function_list.SetItemData(item_index, item_key)
            self.profile_ref_map[item_key] = weakref.ref(profile)
            self.itemDataMap[item_key] = [profile.object_name, profile.function_name, "", "", "", "", ""]
            if profile.is_active():
                self.function_list.CheckItem(item_index)
            item_key += 1
            
        self.function_list.SetColumnWidth(0, wx.LIST_AUTOSIZE)
        self.function_list.SetColumnWidth(1, wx.LIST_AUTOSIZE)

    def UpdateTimer(self, event):
        if self.refresh_ctrl.GetValue() != self.UpdateStatTimer.GetInterval():
            self.UpdateStatTimer.Stop()
            self.UpdateStatTimer = wx.Timer(self)
            self.UpdateStatTimer.Start(self.refresh_ctrl.GetValue())
        self.perf_graph.UpdateGraph()
        self.UpdateData()
        
    def UpdateData(self):
        self.frame_average = self.frame_average_ctrl.GetValue()
        frame_time = profiler.FrameTimeAverage(self.frame_average)
        self.frame_time.SetValue("%.1f ms" % (frame_time*1000.0))
        if frame_time > 0:
            fps = 1.0/frame_time
            self.fps_display.SetValue("%.1f" % fps)
        else:
            self.fps_display.SetValue("-")
        
        if not self.ListIsValid():
            self.InitializeData()
        for item_index in xrange(self.function_list.GetItemCount()):
            item_key = self.function_list.GetItemData(item_index)
            profile = self.profile_ref_map[item_key]()

            if profile.is_active():
                call_time = profile.call_avg(self.frame_average)
                total_time = profile.total_avg(self.frame_average)*1000.0
                owned = profile.owned_avg(self.frame_average)
                owned_time = owned*1000.0
                if frame_time > 0:
                    frame_pct = 100.0*owned/frame_time
                else:
                    frame_pct = 0
                frame_max = profile.frame_max

                self.itemDataMap[item_key][2] = call_time
                self.function_list.SetStringItem(item_index, 2, "%.1f" % call_time)
                self.itemDataMap[item_key][3] = total_time
                self.function_list.SetStringItem(item_index, 3, "%.1f ms" % total_time)
                self.itemDataMap[item_key][4] = owned_time
                self.function_list.SetStringItem(item_index, 4, "%.1f ms" % owned_time)
                self.itemDataMap[item_key][5] = frame_pct
                self.function_list.SetStringItem(item_index, 5, "%.1f%%" % frame_pct)
                self.itemDataMap[item_key][5] = frame_max
                self.function_list.SetStringItem(item_index, 6, "%.3fs" % frame_max)
            else:
                for i in xrange(2, self.function_list.GetColumnCount()):
                    self.itemDataMap[item_key][i] = 0
                    self.function_list.SetStringItem(item_index, i, '-')
            
        
    def ListIsValid(self):
        if self.function_list.GetItemCount() != len(profiler.FunctionProfiles):
            return False
        for key in self.profile_ref_map:
            if self.profile_ref_map[key]() == None:
                return False
        return True

    def CheckItem(self, item_index, flag):
        item_key = self.function_list.GetItemData(item_index)
        profile = self.profile_ref_map[item_key]()
        if profile != None:
            if flag:
                profile.activate()
            else:
                profile.deactivate()
                    
    #---------------------------------------------
    def ShowFunctionBrowser(self, event):
        browser = ModuleBrowser(self.GetParent())
        browser.Show()
        
    #---------------------------------------------
    def RemoveFunction(self, event):
        selected_item = self.function_list.GetFirstSelected()
        if selected_item == -1:
            self.RemoveButton.Enable(False)
        else:
            item_key = self.function_list.GetItemData(selected_item)
            profile = self.profile_ref_map[item_key]()
            if (profile != None):
                profiler.UnregisterFunction(profile.parent_obj, profile.function_name)

    #---------------------------------------------
    def OnItemSelected(self, event):
        self.RemoveButton.Enable(True)

    def OnItemDeselected(self, event):
        self.RemoveButton.Enable(False)
        
########################################################################
class TimingGraph(wx.Frame):
    """ A window that lets you view performance info
        """
    def __init__(self, parent = None):
        num_samples = profiler.FunctionTracker.SampleCount
        wx.Frame.__init__(self, parent, -1, 'TimingGraph', size = (min(num_samples + 100, 600), 400),
                          style=wx.DEFAULT_FRAME_STYLE | wx.NO_FULL_REPAINT_ON_RESIZE)
        self.Bind(wx.EVT_CLOSE, self.OnCloseWindow)

        self.sort_panel = SortableFunctionListPanel(self)

        self.SetSize(self.GetBestSize())
        
    #---------------------------------------------
    def OnCloseWindow(self, event):
        self.Destroy()
        
########################################################################
if __name__ == '__main__':

    import time
    import random
    
    class _TestApp(wx.App):
        def __init__(self):
            profiler.RegisterFunction(self, "Timer10msCallback").activate()
            profiler.RegisterFunction(self, "RandomWait0to5ms").activate()
            self.Window = None
            wx.App.__init__(self, redirect=False)
            self.Bind(wx.EVT_TIMER, self.OnFrameTimer)
            self.FrameTimer = wx.Timer(self)
            self.FrameTimer.Start(66.66)
            self.Timer10ms = wx.Timer(self)
            self.Timer10ms.Start(10)
            
        def OnInit(self):
            self.Window = TimingGraph(None)
            self.Window.Show()
            return True
        
        def OnFrameTimer(self, event):
            if (event.GetInterval() == self.FrameTimer.GetInterval()):
                profiler.AdvanceFrame()
            elif (event.GetInterval() == self.Timer10ms.GetInterval()):
                self.Timer10msCallback()
        
        def Timer10msCallback(self):
            self.RandomWait0to5ms()
        
        def RandomWait0to5ms(self):
            time.sleep(random.random()*.005)
            
    profiler.RegisterFunction(SortableFunctionListPanel, "UpdateData").activate()
    profiler.RegisterFunction(profiler.FunctionTracker, "total_avg").activate()
    profiler.RegisterFunction(PerformanceGraphPanel, "DrawPerfLines").activate()
    app = _TestApp()
    app.MainLoop()
    
        